==> devextreme-20.2.1-alpha-20212-0316.tgz: package/renovation/ui/pager/resizable_container.js <==
/**
 * DevExtreme (renovation/ui/pager/resizable_container.js)
 * Version: 20.2.0 (build 20212-0316)
 * Build date: Thu Jul 30 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";

function _typeof(obj) {
    "@babel/helpers - typeof";
    if ("function" === typeof Symbol && "symbol" === typeof Symbol.iterator) {
        _typeof = function(obj) {
            return typeof obj
        }
    } else {
        _typeof = function(obj) {
            return obj && "function" === typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj
        }
    }
    return _typeof(obj)
}
exports.getContentProps = getContentProps;
exports.updateChildProps = updateChildProps;
exports.ResizableContainer = ResizableContainer;
exports.ResizableContainerProps = exports.viewFunction = void 0;
var _resize_callbacks = _interopRequireDefault(require("../../../core/utils/resize_callbacks"));
var _pager_props = _interopRequireDefault(require("./common/pager_props"));
var _get_element_width = require("./utils/get_element_width");
var Preact = _interopRequireWildcard(require("preact"));
var _hooks = require("preact/hooks");

function _getRequireWildcardCache() {
    if ("function" !== typeof WeakMap) {
        return null
    }
    var cache = new WeakMap;
    _getRequireWildcardCache = function() {
        return cache
    };
    return cache
}

function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
        return obj
    }
    if (null === obj || "object" !== _typeof(obj) && "function" !== typeof obj) {
        return {
            "default": obj
        }
    }
    var cache = _getRequireWildcardCache();
    if (cache && cache.has(obj)) {
        return cache.get(obj)
    }
    var newObj = {};
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc)
            } else {
                newObj[key] = obj[key]
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj)
    }
    return newObj
}

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest()
}

function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")
}

function _unsupportedIterableToArray(o, minLen) {
    if (!o) {
        return
    }
    if ("string" === typeof o) {
        return _arrayLikeToArray(o, minLen)
    }
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if ("Object" === n && o.constructor) {
        n = o.constructor.name
    }
    if ("Map" === n || "Set" === n) {
        return Array.from(o)
    }
    if ("Arguments" === n || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) {
        return _arrayLikeToArray(o, minLen)
    }
}

function _arrayLikeToArray(arr, len) {
    if (null == len || len > arr.length) {
        len = arr.length
    }
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
        arr2[i] = arr[i]
    }
    return arr2
}

function _iterableToArrayLimit(arr, i) {
    if ("undefined" === typeof Symbol || !(Symbol.iterator in Object(arr))) {
        return
    }
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = void 0;
    try {
        for (var _s, _i = arr[Symbol.iterator](); !(_n = (_s = _i.next()).done); _n = true) {
            _arr.push(_s.value);
            if (i && _arr.length === i) {
                break
            }
        }
    } catch (err) {
        _d = true;
        _e = err
    } finally {
        try {
            if (!_n && null != _i.return) {
                _i.return()
            }
        } finally {
            if (_d) {
                throw _e
            }
        }
    }
    return _arr
}

function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) {
        return arr
    }
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}

function _objectWithoutProperties(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for (i = 0; i < sourceSymbolKeys.length; i++) {
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) {
                continue
            }
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) {
                continue
            }
            target[key] = source[key]
        }
    }
    return target
}

function _objectWithoutPropertiesLoose(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) {
            continue
        }
        target[key] = source[key]
    }
    return target
}

function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);
    if (Object.getOwnPropertySymbols) {
        var symbols = Object.getOwnPropertySymbols(object);
        if (enumerableOnly) {
            symbols = symbols.filter(function(sym) {
                return Object.getOwnPropertyDescriptor(object, sym).enumerable
            })
        }
        keys.push.apply(keys, symbols)
    }
    return keys
}

function _objectSpread(target) {
    for (var i = 1; i < arguments.length; i++) {
        var source = null != arguments[i] ? arguments[i] : {};
        if (i % 2) {
            ownKeys(Object(source), true).forEach(function(key) {
                _defineProperty(target, key, source[key])
            })
        } else {
            if (Object.getOwnPropertyDescriptors) {
                Object.defineProperties(target, Object.getOwnPropertyDescriptors(source))
            } else {
                ownKeys(Object(source)).forEach(function(key) {
                    Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key))
                })
            }
        }
    }
    return target
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}
var viewFunction = function(_ref) {
    var infoTextRef = _ref.infoTextRef,
        infoTextVisible = _ref.infoTextVisible,
        isLargeDisplayMode = _ref.isLargeDisplayMode,
        pageSizesRef = _ref.pageSizesRef,
        pagerProps = _ref.pagerProps,
        pagesRef = _ref.pagesRef,
        parentRef = _ref.parentRef,
        Content = _ref.props.contentTemplate,
        restAttributes = _ref.restAttributes;
    return Content(_objectSpread({
        parentRef: parentRef,
        pageSizesRef: pageSizesRef,
        infoTextRef: infoTextRef,
        pagesRef: pagesRef,
        infoTextVisible: infoTextVisible,
        isLargeDisplayMode: isLargeDisplayMode
    }, _objectSpread(_objectSpread({}, pagerProps), restAttributes)))
};
exports.viewFunction = viewFunction;

function getContentProps(_ref2) {
    var infoWidth = _ref2.info,
        pageSizesWidth = _ref2.pageSizes,
        pagesWidth = _ref2.pages,
        parentWidth = _ref2.parent;
    var minimalWidth = pageSizesWidth + pagesWidth;
    var infoTextVisible = parentWidth - minimalWidth > 0;
    var isLargeDisplayMode = parentWidth - (pageSizesWidth + (pagesWidth - infoWidth)) > 0;
    return {
        infoTextVisible: infoTextVisible,
        isLargeDisplayMode: isLargeDisplayMode
    }
}

function getElementsWidth(_ref3) {
    var info = _ref3.info,
        pageSizes = _ref3.pageSizes,
        pages = _ref3.pages,
        parent = _ref3.parent;
    var parentWidth = (0, _get_element_width.getElementWidth)(parent);
    var pageSizesWidth = (0, _get_element_width.getElementWidth)(pageSizes);
    var infoWidth = (0, _get_element_width.getElementWidth)(info);
    var pagesHtmlWidth = (0, _get_element_width.getElementWidth)(pages);
    return {
        parent: parentWidth,
        pageSizes: pageSizesWidth,
        info: infoWidth,
        pages: pagesHtmlWidth
    }
}

function updateElementsWidthIfNeed(elementsWidth, currentElementsWidth) {
    var updated = {
        pageSizes: Math.max(elementsWidth.pageSizes, currentElementsWidth.pageSizes),
        info: Math.max(elementsWidth.info, currentElementsWidth.info),
        pages: Math.max(elementsWidth.pages, currentElementsWidth.pages)
    };
    var isEqual = elementsWidth.pageSizes === updated.pageSizes && elementsWidth.pages === updated.pages && elementsWidth.info === updated.info;
    return isEqual ? elementsWidth : updated
}

function updateChildProps(parentRef, pageSizesHtmlRef, infoTextRef, pagesRef, elementsWidth) {
    var _getElementsWidth = getElementsWidth({
            parent: parentRef,
            pageSizes: null === pageSizesHtmlRef || void 0 === pageSizesHtmlRef ? void 0 : pageSizesHtmlRef.getHtmlElement(),
            info: null === infoTextRef || void 0 === infoTextRef ? void 0 : infoTextRef.getHtmlElement(),
            pages: pagesRef
        }),
        parentWidth = _getElementsWidth.parent,
        currentElementsWidth = _objectWithoutProperties(_getElementsWidth, ["parent"]);
    var newElementsWidth = updateElementsWidthIfNeed(elementsWidth, currentElementsWidth);
    var _getContentProps = getContentProps(_objectSpread({
            parent: parentWidth
        }, newElementsWidth)),
        infoTextVisible = _getContentProps.infoTextVisible,
        isLargeDisplayMode = _getContentProps.isLargeDisplayMode;
    return {
        elementsWidth: newElementsWidth,
        infoTextVisible: infoTextVisible,
        isLargeDisplayMode: isLargeDisplayMode
    }
}
var ResizableContainerProps = _objectSpread({}, _pager_props.default);
exports.ResizableContainerProps = ResizableContainerProps;

function getTemplate(props, template, render, component) {
    var getRender = function(render) {
        return function(props) {
            return "data" in props ? render(props.data, props.index) : render(props)
        }
    };
    var Component = props[component];
    return props[template] || props[render] && getRender(props[render]) || Component && function(props) {
        return Preact.h(Component, _extends({}, props))
    }
}

function ResizableContainer(props) {
    var parentRef = (0, _hooks.useRef)();
    var pageSizesRef = (0, _hooks.useRef)();
    var infoTextRef = (0, _hooks.useRef)();
    var pagesRef = (0, _hooks.useRef)();
    var _useState = (0, _hooks.useState)(function() {
            return void 0 !== props.pageIndex ? props.pageIndex : props.defaultPageIndex
        }),
        _useState2 = _slicedToArray(_useState, 2),
        __state_pageIndex = _useState2[0];
    _useState2[1];
    var _useState3 = (0, _hooks.useState)(function() {
            return void 0 !== props.pageSize ? props.pageSize : props.defaultPageSize
        }),
        _useState4 = _slicedToArray(_useState3, 2),
        __state_pageSize = _useState4[0];
    _useState4[1];
    var _useState5 = (0, _hooks.useState)(true),
        _useState6 = _slicedToArray(_useState5, 2),
        __state_infoTextVisible = _useState6[0],
        __state_setInfoTextVisible = _useState6[1];
    var _useState7 = (0, _hooks.useState)(true),
        _useState8 = _slicedToArray(_useState7, 2),
        __state_isLargeDisplayMode = _useState8[0],
        __state_setIsLargeDisplayMode = _useState8[1];
    var _useState9 = (0, _hooks.useState)(-1),
        _useState10 = _slicedToArray(_useState9, 2),
        __state_parentWidth = _useState10[0];
    _useState10[1];
    var _useState11 = (0, _hooks.useState)({
            pageSizes: 0,
            pages: 0,
            info: 0
        }),
        _useState12 = _slicedToArray(_useState11, 2),
        __state_elementsWidth = _useState12[0],
        __state_setElementsWidth = _useState12[1];
    var __pagerProps = (0, _hooks.useCallback)(function() {
        var _props$pageIndex$page = _objectSpread(_objectSpread({}, props), {}, {
                pageIndex: void 0 !== props.pageIndex ? props.pageIndex : __state_pageIndex,
                pageSize: void 0 !== props.pageSize ? props.pageSize : __state_pageSize
            }),
            pagerProps = (_props$pageIndex$page.contentTemplate, _objectWithoutProperties(_props$pageIndex$page, ["contentTemplate"]));
        return pagerProps
    }, [props, __state_pageIndex, __state_pageSize]);
    var updateChildrenProps = (0, _hooks.useCallback)(function() {
        var _updateChildProps = updateChildProps(parentRef.current, pageSizesRef.current, infoTextRef.current, pagesRef.current, __state_elementsWidth),
            elementsWidth = _updateChildProps.elementsWidth,
            infoTextVisible = _updateChildProps.infoTextVisible,
            isLargeDisplayMode = _updateChildProps.isLargeDisplayMode;
        __state_setElementsWidth(function(__state_elementsWidth) {
            return elementsWidth
        });
        __state_setInfoTextVisible(function(__state_infoTextVisible) {
            return infoTextVisible
        });
        __state_setIsLargeDisplayMode(function(__state_isLargeDisplayMode) {
            return isLargeDisplayMode
        })
    }, [pageSizesRef.current, infoTextRef.current, __state_elementsWidth]);
    var __restAttributes = (0, _hooks.useCallback)(function() {
        var _props$pageIndex$page2 = _objectSpread(_objectSpread({}, props), {}, {
                pageIndex: void 0 !== props.pageIndex ? props.pageIndex : __state_pageIndex,
                pageSize: void 0 !== props.pageSize ? props.pageSize : __state_pageSize
            }),
            restProps = (_props$pageIndex$page2.className, _props$pageIndex$page2.contentComponent, _props$pageIndex$page2.contentRender, _props$pageIndex$page2.contentTemplate, _props$pageIndex$page2.defaultPageIndex, _props$pageIndex$page2.defaultPageSize, _props$pageIndex$page2.hasKnownLastPage, _props$pageIndex$page2.infoText, _props$pageIndex$page2.lightModeEnabled, _props$pageIndex$page2.maxPagesCount, _props$pageIndex$page2.pageCount, _props$pageIndex$page2.pageIndex, _props$pageIndex$page2.pageIndexChange, _props$pageIndex$page2.pageSize, _props$pageIndex$page2.pageSizeChange, _props$pageIndex$page2.pageSizes, _props$pageIndex$page2.pagesCountText, _props$pageIndex$page2.pagesNavigatorVisible, _props$pageIndex$page2.rtlEnabled, _props$pageIndex$page2.showInfo, _props$pageIndex$page2.showNavigationButtons, _props$pageIndex$page2.showPageSizes, _props$pageIndex$page2.totalCount, _props$pageIndex$page2.visible, _objectWithoutProperties(_props$pageIndex$page2, ["className", "contentComponent", "contentRender", "contentTemplate", "defaultPageIndex", "defaultPageSize", "hasKnownLastPage", "infoText", "lightModeEnabled", "maxPagesCount", "pageCount", "pageIndex", "pageIndexChange", "pageSize", "pageSizeChange", "pageSizes", "pagesCountText", "pagesNavigatorVisible", "rtlEnabled", "showInfo", "showNavigationButtons", "showPageSizes", "totalCount", "visible"]));
        return restProps
    }, [props, __state_pageIndex, __state_pageSize]);
    (0, _hooks.useEffect)(function() {
        var callback = function() {
            return updateChildrenProps()
        };
        _resize_callbacks.default.add(callback);
        return function() {
            _resize_callbacks.default.remove(callback)
        }
    }, []);
    (0, _hooks.useEffect)(function() {
        var parentWidth = (0, _get_element_width.getElementWidth)(parentRef.current);
        if (__state_parentWidth === -1 && parentWidth > 0 || __state_parentWidth === parentWidth) {
            updateChildrenProps()
        }
    }, [__state_parentWidth, pageSizesRef.current, infoTextRef.current, __state_elementsWidth]);
    return viewFunction({
        props: _objectSpread(_objectSpread({}, props), {}, {
            pageIndex: void 0 !== props.pageIndex ? props.pageIndex : __state_pageIndex,
            pageSize: void 0 !== props.pageSize ? props.pageSize : __state_pageSize,
            contentTemplate: getTemplate(props, "contentTemplate", "contentRender", "contentComponent")
        }),
        infoTextVisible: __state_infoTextVisible,
        isLargeDisplayMode: __state_isLargeDisplayMode,
        parentWidth: __state_parentWidth,
        elementsWidth: __state_elementsWidth,
        parentRef: parentRef,
        pageSizesRef: pageSizesRef,
        infoTextRef: infoTextRef,
        pagesRef: pagesRef,
        pagerProps: __pagerProps(),
        updateChildrenProps: updateChildrenProps,
        restAttributes: __restAttributes()
    })
}
ResizableContainer.defaultProps = _objectSpread({}, ResizableContainerProps);

==> devextreme-20.2.2-alpha-20232-1710.tgz: package/renovation/ui/pager/resizable_container.js <==
/**
 * DevExtreme (renovation/ui/pager/resizable_container.js)
 * Version: 20.2.1 (build 20232-1710)
 * Build date: Wed Aug 19 2020
 *
 * Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";

function _typeof(obj) {
    "@babel/helpers - typeof";
    if ("function" === typeof Symbol && "symbol" === typeof Symbol.iterator) {
        _typeof = function(obj) {
            return typeof obj
        }
    } else {
        _typeof = function(obj) {
            return obj && "function" === typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj
        }
    }
    return _typeof(obj)
}
exports.getContentProps = getContentProps;
exports.updateChildProps = updateChildProps;
exports.ResizableContainer = ResizableContainer;
exports.ResizableContainerProps = exports.viewFunction = void 0;
var _resize_callbacks = _interopRequireDefault(require("../../../core/utils/resize_callbacks"));
var _get_element_width = require("./utils/get_element_width");
var Preact = _interopRequireWildcard(require("preact"));
var _hooks = require("preact/hooks");

function _getRequireWildcardCache() {
    if ("function" !== typeof WeakMap) {
        return null
    }
    var cache = new WeakMap;
    _getRequireWildcardCache = function() {
        return cache
    };
    return cache
}

function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
        return obj
    }
    if (null === obj || "object" !== _typeof(obj) && "function" !== typeof obj) {
        return {
            "default": obj
        }
    }
    var cache = _getRequireWildcardCache();
    if (cache && cache.has(obj)) {
        return cache.get(obj)
    }
    var newObj = {};
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc)
            } else {
                newObj[key] = obj[key]
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj)
    }
    return newObj
}

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest()
}

function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")
}

function _unsupportedIterableToArray(o, minLen) {
    if (!o) {
        return
    }
    if ("string" === typeof o) {
        return _arrayLikeToArray(o, minLen)
    }
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if ("Object" === n && o.constructor) {
        n = o.constructor.name
    }
    if ("Map" === n || "Set" === n) {
        return Array.from(o)
    }
    if ("Arguments" === n || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) {
        return _arrayLikeToArray(o, minLen)
    }
}

function _arrayLikeToArray(arr, len) {
    if (null == len || len > arr.length) {
        len = arr.length
    }
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
        arr2[i] = arr[i]
    }
    return arr2
}

function _iterableToArrayLimit(arr, i) {
    if ("undefined" === typeof Symbol || !(Symbol.iterator in Object(arr))) {
        return
    }
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = void 0;
    try {
        for (var _s, _i = arr[Symbol.iterator](); !(_n = (_s = _i.next()).done); _n = true) {
            _arr.push(_s.value);
            if (i && _arr.length === i) {
                break
            }
        }
    } catch (err) {
        _d = true;
        _e = err
    } finally {
        try {
            if (!_n && null != _i.return) {
                _i.return()
            }
        } finally {
            if (_d) {
                throw _e
            }
        }
    }
    return _arr
}

function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) {
        return arr
    }
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}

function _objectWithoutProperties(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for (i = 0; i < sourceSymbolKeys.length; i++) {
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) {
                continue
            }
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) {
                continue
            }
            target[key] = source[key]
        }
    }
    return target
}

function _objectWithoutPropertiesLoose(source, excluded) {
    if (null == source) {
        return {}
    }
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for (i = 0; i < sourceKeys.length; i++) {
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) {
            continue
        }
        target[key] = source[key]
    }
    return target
}

function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);
    if (Object.getOwnPropertySymbols) {
        var symbols = Object.getOwnPropertySymbols(object);
        if (enumerableOnly) {
            symbols = symbols.filter(function(sym) {
                return Object.getOwnPropertyDescriptor(object, sym).enumerable
            })
        }
        keys.push.apply(keys, symbols)
    }
    return keys
}

function _objectSpread(target) {
    for (var i = 1; i < arguments.length; i++) {
        var source = null != arguments[i] ? arguments[i] : {};
        if (i % 2) {
            ownKeys(Object(source), true).forEach(function(key) {
                _defineProperty(target, key, source[key])
            })
        } else {
            if (Object.getOwnPropertyDescriptors) {
                Object.defineProperties(target, Object.getOwnPropertyDescriptors(source))
            } else {
                ownKeys(Object(source)).forEach(function(key) {
                    Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key))
                })
            }
        }
    }
    return target
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        })
    } else {
        obj[key] = value
    }
    return obj
}
var viewFunction = function(_ref) {
    var infoTextRef = _ref.infoTextRef,
        infoTextVisible = _ref.infoTextVisible,
        isLargeDisplayMode = _ref.isLargeDisplayMode,
        pageSizesRef = _ref.pageSizesRef,
        pagesRef = _ref.pagesRef,
        parentRef = _ref.parentRef,
        _ref$props = _ref.props,
        Content = _ref$props.contentTemplate,
        pagerProps = _ref$props.pagerProps,
        restAttributes = _ref.restAttributes;
    return Content(_objectSpread({
        parentRef: parentRef,
        pageSizesRef: pageSizesRef,
        infoTextRef: infoTextRef,
        pagesRef: pagesRef,
        infoTextVisible: infoTextVisible,
        isLargeDisplayMode: isLargeDisplayMode
    }, _objectSpread(_objectSpread({}, pagerProps), restAttributes)))
};
exports.viewFunction = viewFunction;

function getContentProps(_ref2) {
    var infoWidth = _ref2.info,
        pageSizesWidth = _ref2.pageSizes,
        pagesWidth = _ref2.pages,
        parentWidth = _ref2.parent;
    var minimalWidth = pageSizesWidth + pagesWidth;
    var infoTextVisible = parentWidth - minimalWidth > 0;
    var isLargeDisplayMode = parentWidth - (pageSizesWidth + (pagesWidth - infoWidth)) > 0;
    return {
        infoTextVisible: infoTextVisible,
        isLargeDisplayMode: isLargeDisplayMode
    }
}

function getElementsWidth(_ref3) {
    var info = _ref3.info,
        pageSizes = _ref3.pageSizes,
        pages = _ref3.pages,
        parent = _ref3.parent;
    var parentWidth = (0, _get_element_width.getElementWidth)(parent);
    var pageSizesWidth = (0, _get_element_width.getElementWidth)(pageSizes);
    var infoWidth = (0, _get_element_width.getElementWidth)(info);
    var pagesHtmlWidth = (0, _get_element_width.getElementWidth)(pages);
    return {
        parent: parentWidth,
        pageSizes: pageSizesWidth,
        info: infoWidth,
        pages: pagesHtmlWidth
    }
}

function updateElementsWidthIfNeed(elementsWidth, currentElementsWidth) {
    var updated = {
        pageSizes: Math.max(elementsWidth.pageSizes, currentElementsWidth.pageSizes),
        info: Math.max(elementsWidth.info, currentElementsWidth.info),
        pages: Math.max(elementsWidth.pages, currentElementsWidth.pages)
    };
    var isEqual = elementsWidth.pageSizes === updated.pageSizes && elementsWidth.pages === updated.pages && elementsWidth.info === updated.info;
    return isEqual ? elementsWidth : updated
}

function updateChildProps(parentRef, pageSizesHtmlRef, infoTextRef, pagesRef, elementsWidth) {
    var _getElementsWidth = getElementsWidth({
            parent: parentRef,
            pageSizes: null === pageSizesHtmlRef || void 0 === pageSizesHtmlRef ? void 0 : pageSizesHtmlRef.getHtmlElement(),
            info: null === infoTextRef || void 0 === infoTextRef ? void 0 : infoTextRef.getHtmlElement(),
            pages: pagesRef
        }),
        parentWidth = _getElementsWidth.parent,
        currentElementsWidth = _objectWithoutProperties(_getElementsWidth, ["parent"]);
    var newElementsWidth = updateElementsWidthIfNeed(elementsWidth, currentElementsWidth);
    var _getContentProps = getContentProps(_objectSpread({
            parent: parentWidth
        }, newElementsWidth)),
        infoTextVisible = _getContentProps.infoTextVisible,
        isLargeDisplayMode = _getContentProps.isLargeDisplayMode;
    return {
        elementsWidth: newElementsWidth,
        infoTextVisible: infoTextVisible,
        isLargeDisplayMode: isLargeDisplayMode
    }
}
var ResizableContainerProps = {};
exports.ResizableContainerProps = ResizableContainerProps;

function getTemplate(props, template, render, component) {
    var getRender = function(render) {
        return function(props) {
            return "data" in props ? render(props.data, props.index) : render(props)
        }
    };
    var Component = props[component];
    return props[template] || props[render] && getRender(props[render]) || Component && function(props) {
        return Preact.h(Component, _extends({}, props))
    }
}

function ResizableContainer(props) {
    var parentRef = (0, _hooks.useRef)();
    var pageSizesRef = (0, _hooks.useRef)();
    var infoTextRef = (0, _hooks.useRef)();
    var pagesRef = (0, _hooks.useRef)();
    var _useState = (0, _hooks.useState)(true),
        _useState2 = _slicedToArray(_useState, 2),
        __state_infoTextVisible = _useState2[0],
        __state_setInfoTextVisible = _useState2[1];
    var _useState3 = (0, _hooks.useState)(true),
        _useState4 = _slicedToArray(_useState3, 2),
        __state_isLargeDisplayMode = _useState4[0],
        __state_setIsLargeDisplayMode = _useState4[1];
    var _useState5 = (0, _hooks.useState)(-1),
        _useState6 = _slicedToArray(_useState5, 2),
        __state_parentWidth = _useState6[0];
    _useState6[1];
    var _useState7 = (0, _hooks.useState)({
            pageSizes: 0,
            pages: 0,
            info: 0
        }),
        _useState8 = _slicedToArray(_useState7, 2),
        __state_elementsWidth = _useState8[0],
        __state_setElementsWidth = _useState8[1];
    var updateChildrenProps = (0, _hooks.useCallback)(function() {
        var _updateChildProps = updateChildProps(parentRef.current, pageSizesRef.current, infoTextRef.current, pagesRef.current, __state_elementsWidth),
            elementsWidth = _updateChildProps.elementsWidth,
            infoTextVisible = _updateChildProps.infoTextVisible,
            isLargeDisplayMode = _updateChildProps.isLargeDisplayMode;
        __state_setElementsWidth(function(__state_elementsWidth) {
            return elementsWidth
        });
        __state_setInfoTextVisible(function(__state_infoTextVisible) {
            return infoTextVisible
        });
        __state_setIsLargeDisplayMode(function(__state_isLargeDisplayMode) {
            return isLargeDisplayMode
        })
    }, [pageSizesRef.current, infoTextRef.current, __state_elementsWidth]);
    var __restAttributes = (0, _hooks.useCallback)(function() {
        var restProps = (props.contentComponent, props.contentRender, props.contentTemplate, props.pagerProps, _objectWithoutProperties(props, ["contentComponent", "contentRender", "contentTemplate", "pagerProps"]));
        return restProps
    }, [props]);
    (0, _hooks.useEffect)(function() {
        var callback = function() {
            return updateChildrenProps()
        };
        _resize_callbacks.default.add(callback);
        return function() {
            _resize_callbacks.default.remove(callback)
        }
    }, []);
    (0, _hooks.useEffect)(function() {
        var parentWidth = (0, _get_element_width.getElementWidth)(parentRef.current);
        if (__state_parentWidth === -1 && parentWidth > 0 || __state_parentWidth === parentWidth) {
            updateChildrenProps()
        }
    }, [__state_parentWidth, pageSizesRef.current, infoTextRef.current, __state_elementsWidth]);
    return viewFunction({
        props: _objectSpread(_objectSpread({}, props), {}, {
            contentTemplate: getTemplate(props, "contentTemplate", "contentRender", "contentComponent")
        }),
        infoTextVisible: __state_infoTextVisible,
        isLargeDisplayMode: __state_isLargeDisplayMode,
        parentWidth: __state_parentWidth,
        elementsWidth: __state_elementsWidth,
        parentRef: parentRef,
        pageSizesRef: pageSizesRef,
        infoTextRef: infoTextRef,
        pagesRef: pagesRef,
        updateChildrenProps: updateChildrenProps,
        restAttributes: __restAttributes()
    })
}
ResizableContainer.defaultProps = _objectSpread({}, ResizableContainerProps);
